#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Constants used for building and more."""

from .log import get_logger

logger = get_logger(__name__)

SAGEMAKER_CONTAINER_PATH_MAIN = "/opt/ml/processing"
"""Path to the main folder in the sagemaker container."""

CONDA_ENV_NAME = "py3710_data_apps"
"""Name of the conda environment to use for data apps project."""

EXCLUDE_FOLDERS = [
    "__pycache__",
    "tests",
    "tests_data",
    "conda",
    "eggs",
    "build",
    "dist",
    ".git",
    "*.egg-info",
]
"""Folders to exclude from the zip, while compressing data apps repository code."""

EXCLUDE_FILES = [
    "*.pyc",
    "*.pyo",
    "*.dist-info",
    "*.DS_Store",
    "*.gitignore",
    "*.gitmodules",
    "*.gitattributes",
    "*.gitkeep",
]
"""Files to exclude from the zip, while compressing data apps repository code."""



"""
Running queries on the model registry.
"""
import tempfile

from rudderlabs.data.apps.aws.s3 import (
    download_s3_directory,
    get_s3_resource,
    parse_s3_path,
)
from rudderlabs.data.apps.wh import Connector


class ModelLoader:
    def __init__(self, creds_config: dict) -> None:
        self.creds_config = creds_config

        # Expect table information(name, schema, database) from data_warehouse section of
        # credentials configurations
        self.database = creds_config["data_warehouse"]["database"]
        self.schema = creds_config["data_warehouse"]["schema"]
        self.model_registry_table = creds_config["data_warehouse"][
            "model_registry_table"
        ]

    def get_connector(self):
        """Returns a connector object."""
        warehouse_creds = self.creds_config["data_warehouse"]
        aws_config = self.creds_config["aws"]

        wh_connector = Connector(warehouse_creds, aws_config=aws_config)
        return wh_connector

    def get_latest_model(self, model_type, job_id=None) -> dict:
        """Returns the latest model of a given type.

        Args:
            model_type: The type of model to return.
            job_id: The job_id of the model to return.

        Returns:
            dict: The latest model of the given type.
        """

        # Get the latest model staged in the model registry
        # Prepare where condition
        where_condition = f"model_type = '{model_type}'"
        if job_id:
            where_condition += f" and job_id = '{job_id}'"

        query = f"""
            SELECT * FROM {self.database}.{self.schema}.{self.model_registry_table}
            WHERE {where_condition}
            ORDER BY timestamp DESC
            LIMIT 1
        """

        connector = self.get_connector()
        df = connector.run_query(query)
        model_data = df.to_dict(orient="records")

        if len(model_data):
            model_data = model_data[0]

        return model_data

    def download_model_files_to_temp(self, model_data: dict) -> None:
        """Downloads the model files from S3 to local path.

        Args:
            model_data (dict): The model data.
            local_path (str): The local path to download the model files to.
        """
        if not model_data:
            return None

        temp_folder = tempfile.mkdtemp()
        print(f"Downloading model data to temporary location {temp_folder}")

        s3_bucket, s3_prefix = parse_s3_path(model_data["model_files_location"])
        s3_resource = get_s3_resource(self.creds_config)

        download_s3_directory(s3_resource, s3_bucket, s3_prefix, temp_folder)
        return temp_folder



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""SnowFlake connector."""


import pandas as pd

from sqlalchemy import create_engine

from ..log import get_logger
from .connector_base import ConnectorBase, register_connector

logger = get_logger(__name__)


@register_connector
class SnowflakeConnector(ConnectorBase):
    def __init__(self, creds: dict, db_config: dict, **kwargs) -> None:
        super().__init__(creds, db_config, **kwargs)
        url = f"snowflake://{creds['user']}:{creds['password']}@{creds['account_identifier']}"
        if "database" in db_config:
            url += f"/{db_config['database']}"
            if "schema" in db_config:
                url += f"/{db_config['schema']}"
                if "warehouse" in creds:
                    url += f"?warehouse={creds['warehouse']}"
                    if "role" in creds:
                        url += f"&role={creds['role']}"
        self.engine = create_engine(url)
        self.connection = self.engine.connect()

    def write_to_table(
        self,
        df: pd.DataFrame,
        table_name: str,
        schema: str = None,
        if_exists: str = "append",
    ):
        table_name, schema = (
            table_name.split(".") if "." in table_name else (table_name, schema)
        )
        print("Writing to table: {}.{}".format(schema, table_name))
        df.to_sql(
            table_name,
            self.engine,
            schema=schema,
            if_exists=if_exists,
            index=False,
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Compress and decompress a directory."""

import os
import tempfile
import zipfile

from fnmatch import fnmatch

from ..log import get_logger

TEMP_DIR = tempfile.gettempdir()
logger = get_logger(__name__)


def can_exclude(name: str, exclude_list: list) -> bool:
    """Check if the name can be excluded

    Args:
        name: Name to exclude
        exlude_list: List of names to be excluded can be regular expressions

    Returns:
        bool: True if the name can be excluded
    """
    for exclude_name in exclude_list:
        if fnmatch(name, exclude_name):
            return True
    return False


def zip_directory(
    dir_path: str, exclude_folders: list = [], exclude_files: list = []
) -> str:
    """Zip the directory

    Args:
        dir_path (str):  Path to the directory to zip
        exclude_folders (list): List of directories to be excluded from the zip
        exclude_files (list): List of files to be excluded from the zip

    Returns:
        str: Path to the zip file

    Raises:
        ValueError: If the directory does not exist
    """
    logger.info("Zipping directory")
    if not os.path.exists(dir_path):
        raise ValueError(f"Directory {dir_path} does not exist")

    dir_name = os.path.basename(dir_path)
    zip_path = os.path.join(TEMP_DIR, dir_name + ".zip")

    with zipfile.ZipFile(zip_path, "w", zipfile.ZIP_DEFLATED) as zip_file:
        for root, dirs, files in os.walk(dir_path):

            # Exclude folders
            for dirname in dirs:
                if can_exclude(dirname, exclude_folders):
                    dirs.remove(dirname)

            for file in files:
                # Exclude files
                if can_exclude(file, exclude_files):
                    continue

                abs_file_path = os.path.join(root, file)
                zip_file.write(
                    abs_file_path, abs_file_path.replace(dir_path, "")
                )

    return zip_path


def unzip_directory(zip_path: str, out_dir_path: str) -> None:
    """Unzip the directory

    Args:
        zip_path: Path to the zipped directory
        dir_path: Output directory path

    Returns:
        None: None

    Raises:
        ValueError: If the zip file does not exist
        ValueError: If the output directory does not exist
    """
    logger.info("Unzipping directory")
    if not os.path.exists(zip_path):
        raise ValueError(f"Zip file {zip_path} does not exist")

    if not os.path.exists(out_dir_path):
        raise ValueError(f"Output directory {out_dir_path} does not exist")

    with zipfile.ZipFile(zip_path, "r") as zip_file:
        zip_file.extractall(out_dir_path)
